@@ -91,7 +91,7 @@ U = TypeVar("U")
 
 class ModelBase(Mapping[str, Any]):
     _column_info: Optional[Dict[str, ColumnInfo]]
-    _fragment_cache: Optional[Dict[tuple, Any]]
+    _cache: Optional[Dict[tuple, Any]]
     table_name: str
     primary_key_names: Tuple[str, ...]
 
@@ -107,12 +107,12 @@ class ModelBase(Mapping[str, Any]):
     @classmethod
     def _cached(cls, key: tuple, thunk: Callable[[], U]) -> U:
         try:
-            fragment_cache: Dict[tuple, Any] = cls._fragment_cache  # type: ignore
+            cache: Dict[tuple, Any] = cls._cache  # type: ignore
         except AttributeError:
-            fragment_cache = cls._fragment_cache = {}
-        if key not in fragment_cache:
-            fragment_cache[key] = thunk()
-        return fragment_cache[key]
+            cache = cls._cache = {}
+        if key not in cache:
+            cache[key] = thunk()
+        return cache[key]
 
     def keys(self):
         return self.field_names()
@@ -160,24 +160,34 @@ class ModelBase(Mapping[str, Any]):
     def primary_key(self) -> tuple:
         return tuple(getattr(self, pk) for pk in self.primary_key_names)
 
+    @classmethod
+    def _get_field_values_fn(cls, exclude: FieldNamesSet = ()):
+        env: Dict[str, Any] = dict()
+        func = ["def get_field_values(self):", " return ["]
+        for f in fields(cls):
+            if f.name not in exclude:
+                func.append(f"  self.{f.name},")
+        func += [" ]"]
+        exec("\n".join(func), env)
+        return env["get_field_values"]
+
     def field_values(self, *, exclude: FieldNamesSet = ()) -> List[Any]:
-        return [getattr(self, f.name) for f in fields(self) if f.name not in exclude]
+        get_field_values = self._cached(
+            ("get_field_values", tuple(sorted(exclude))),
+            lambda: self._get_field_values_fn(exclude),
+        )
+        return get_field_values(self)
 
     def field_values_sql(
         self, *, exclude: FieldNamesSet = (), default_none=False
     ) -> List[Fragment]:
         if default_none:
-
-            def field_value(name):
-                value = getattr(self, name)
-                return sql.literal("DEFAULT") if value is None else sql.value(value)
-
+            return [
+                sql.literal("DEFAULT") if value is None else sql.value(value)
+                for value in self.field_values()
+            ]
         else:
-
-            def field_value(name):
-                return sql.value(getattr(self, name))
-
-        return [field_value(f.name) for f in fields(self) if f.name not in exclude]
+            return [sql.value(value) for value in self.field_values()]
 
     @classmethod
     def from_tuple(
@@ -233,7 +243,7 @@ class ModelBase(Mapping[str, Any]):
         )
         query = cached(where=where)
         if for_update:
-            query = Fragment([query, " FOR UPDATE"])
+            query = Fragment([query, " FOR UPDATE"], {})
         return query
 
     @classmethod
@@ -303,7 +313,7 @@ class ModelBase(Mapping[str, Any]):
                 ),
             ).flatten(),
         )
-        return Fragment([insert_sql, cached])
+        return Fragment([insert_sql, cached], {})
 
     async def upsert(self, connection_or_pool, exclude: FieldNamesSet = ()) -> bool:
         query = sql(
@@ -367,7 +377,7 @@ class ModelBase(Mapping[str, Any]):
         rows: Union[Iterable[T], Iterable[Mapping[str, Any]]],
         *,
         where: Where,
-        ignore: FieldNamesSet = ()
+        ignore: FieldNamesSet = (),
     ) -> Tuple[List[T], List[T], List[T]]:
         pending = {row.primary_key(): row for row in map(cls.ensure_model, rows)}
 
